@@ -3,7 +3,7 @@ package Net::Async::Slack::Socket;
 use strict;
 use warnings;
 
-our $VERSION = '0.007'; # VERSION
+our $VERSION = '0.008'; # VERSION
 our $AUTHORITY = 'cpan:TEAM'; # AUTHORITY
 
 use parent qw(IO::Async::Notifier);
@@ -138,6 +138,7 @@ use Net::Async::Slack::Event::UserResourceGranted;
 use Net::Async::Slack::Event::UserResourceRemoved;
 use Net::Async::Slack::Event::UserTyping;
 
+use List::Util qw(min);
 use Log::Any qw($log);
 
 my $json = JSON::MaybeXS->new;
@@ -178,17 +179,23 @@ Establishes the connection. Called by the top-level L<Net::Async::Slack> instanc
 
 async sub connect {
     my ($self, %args) = @_;
-    my $uri = $self->wss_uri or die 'no websocket URI available';
+    my $uri = delete($args{uri}) // $self->wss_uri or die 'no websocket URI available';
+    my $prev = $self->{ws};
     $self->add_child(
         $self->{ws} = Net::Async::WebSocket::Client->new(
             on_frame => $self->curry::weak::on_frame,
             on_ping_frame => $self->curry::weak::on_ping_frame,
+            on_close_frame => $self->curry::weak::on_close_frame,
         )
     );
     $log->tracef('URL for websockets will be %s', "$uri");
     my $res = await $self->{ws}->connect(
         url        => "$uri",
     );
+    if($prev) {
+        $log->tracef('Closing previous websocket connection');
+        $prev->close_now;
+    }
     $self->event_mangler;
     return $res;
 }
@@ -198,6 +205,51 @@ sub on_ping_frame {
     $ws->send_pong_frame('');
 }
 
+sub on_close_frame {
+    my ($self) = @_;
+    $log->debugf('Received close frame');
+    $self->trigger_reconnect_if_needed
+}
+async sub reconnect {
+    my ($self) = @_;
+    my $sleep = 0;
+    my $count = 0;
+    while(1) {
+        ++$count;
+        try {
+            $log->debugf('Attempting reconnect, try %d', $count);
+            my ($uri) = await Future->wait_any(
+                $self->slack->socket,
+                $self->loop->timeout_future(after => 30),
+            );
+            await Future->wait_any(
+                $self->connect(
+                    uri => $uri
+                ),
+                $self->loop->timeout_future(after => 30),
+            );
+            return;
+        } catch($e) {
+            $sleep = min(30.0, ($sleep || 0.008) * 2);
+            $log->errorf('Failed to reconnect for socket mode, will try again in %.3fs: %s', $sleep, $e);
+            await $self->loop->delay_future(after => $sleep);
+        }
+    }
+}
+
+sub on_close {
+    my ($self) = @_;
+    $self->trigger_reconnect_if_needed;
+}
+
+sub trigger_reconnect_if_needed {
+    my ($self) = @_;
+    $log->tracef('trigger_reconnect_if_needed');
+    return $self->{reconnecting} ||= $self->reconnect->on_ready(sub {
+        delete $self->{reconnecting}
+    });
+}
+
 sub on_frame {
     my ($self, $ws, $bytes) = @_;
     my $text = Encode::decode_utf8($bytes);
@@ -209,11 +261,17 @@ sub on_frame {
         $log->tracef("<< %s", $text);
         try {
             my $data = $json->decode($text);
+            if($data->{type} eq 'disconnect') {
+                $log->debugf('Received disconnection notification, reason: %s (debug info: %s)', $data->{reason}, $data->{debug_info});
+                $self->trigger_reconnect_if_needed;
+            }
             if(my $env_id = $data->{envelope_id}) {
+                my $data = $json->encode({
+                    envelope_id => $env_id
+                });
+                $log->tracef(">> %s", $data);
                 $self->ws->send_frame(
-                    buffer => $json->encode({
-                        envelope_id => $env_id
-                    }),
+                    buffer => $data,
                     masked => 1
                 )->retain;
             }
